package com.poo.agusa.usmroute;

import android.util.Log;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Scanner;

public class Ruta {

    private List<Vertex> nodes;
    private List<Edge> edges;
    private DijkstraAlgorithm dijkstra;
    private Graph graph;
    static public LinkedList<Vertex> ruta_actual;

    public Ruta(InputStream a_nodos, InputStream a_rutas)
    {
        nodes = new ArrayList<Vertex>();
        edges = new ArrayList<Edge>();
        Scanner scan = new Scanner(a_nodos);
        while(scan.hasNext())
        {
            String linea = scan.nextLine();
            String[] s_nodos = linea.split(",");

            for(String s : s_nodos)
            {
                Vertex location = new Vertex(s, s);
                nodes.add(location);
                Log.d("USM", "Agregado nodo: " + s);
            }
        }

        scan = new Scanner(a_rutas);
        int laneid = 0;
        while(scan.hasNext()) {
            String linea = scan.nextLine();
            String[] str1 = linea.split(":");

            Log.d("USM", "Nodo inicial: " + str1[0]);

            String[] sub_nodos = str1[1].split("\\|");
            for (String s : sub_nodos) {
                Log.d("USM", "sub_nodo: " + s);

                String[] datos = s.split(",");

                int weight = (datos.length < 1 ) ? Integer.parseInt(datos[1]) : 1;
                addLane(str1[0] + "_" + datos[0], str1[0], datos[0], weight);
            }

        }

        graph = new Graph(nodes, edges);
        dijkstra = new DijkstraAlgorithm(graph);

        LinkedList<Vertex> path = new LinkedList<Vertex>();
        for(Vertex v : nodes)
            if(v.getId().equals("NodoJ"))
                dijkstra.execute(v);

        for(Vertex v : nodes)
            if(v.getId().equals("P101"))
                path = dijkstra.getPath(v);

        // DEBUG
        for (Vertex vertex : path) {
            System.out.println(vertex);
        }
    }

    public LinkedList<Vertex> obtenerRuta(String inicio, String termino)
    {
        LinkedList<Vertex> path = new LinkedList<Vertex>();
        for(Vertex v : nodes)
            if(v.getId().equals(inicio))
                dijkstra.execute(v);

        for(Vertex v : nodes)
            if(v.getId().equals(termino))
                path = dijkstra.getPath(v);

        if (path == null) {
            Log.d("USM", "No se encontro ruta para la sala.");
            return null;
        }

        // DEBUG
        for (Vertex vertex : path) {
            System.out.println(vertex);
        }

        ruta_actual = path;
        return path;
    }

    public boolean existeNodo(String nombre)
    {
        for(Vertex v : nodes)
            if(v.getId().equals(nombre))
                return true;

        return false;
    }

    private void addLane(String laneId, String sourceid, String destid,
                         int duration) {

        Vertex source = new Vertex("a","a");
        Vertex destination = new Vertex("b","b");
        for(Vertex v : nodes)
            if(v.getId().equals(sourceid))
                source = v;

        for(Vertex v : nodes)
            if(v.getId().equals(destid))
                destination = v;


        Edge lane = new Edge(laneId,source, destination, duration );
        edges.add(lane);
    }

    static public ArrayList<String> obtenerDirecciones() {
        ArrayList<String> dirs = new ArrayList<String>();
        Vertex src = new Vertex("","");
        for(Vertex v : ruta_actual)
        {
            if(!src.getId().isEmpty())
                dirs.add(src.getId() + "_" + v.getId());

            src = v;
        }

        return dirs;
    }

}
